@@ -31,6 +31,7 @@ from .sqla import (
     core_coerce_row,
     core_result_type,
     get_bind,
+    get_session,
     group_by_clauses,
     orm_coerce_row,
     orm_query_keys,
@@ -51,22 +52,15 @@ PER_PAGE_DEFAULT = 10
 SUPPORTS_NATIVE_TUPLE_COMPARISON = ("postgresql", "mysql", "sqlite")
 
 
-def compare_tuples(
-    lesser: Sequence, greater: Sequence, dialect: Optional[Dialect] = None
-) -> ColumnElement[bool]:
+def compare_tuples(lesser: Sequence, greater: Sequence) -> ColumnElement[bool]:
     """Given two sequences of equal length (whose entries can be SQL clauses or
-    simple values), create an SQL clause defining the lexicographic tuple
-    comparison ``lesser < greater``.
-
-    If ``dialect`` is provided and is an sqlalchemy SQL dialect supporting
-    native tuple comparison, the SQL emitted is a native tuple comparison.
-    Otherwise it is built manually using OR and AND."""
+    simple values), create a simple SQL clause equivalent to the lexicographic
+    tuple comparison ``lesser < greater``."""
     if len(lesser) != len(greater):
         raise ValueError("Tuples must have same length to be compared!")
-    if len(lesser) == 1:
-        return lesser[0] < greater[0]
-    if dialect is not None and dialect.name.lower() in SUPPORTS_NATIVE_TUPLE_COMPARISON:
-        return tuple_(*lesser) < tuple_(*greater)
+
+    # This monster comprehension is just the long way of writing the
+    # lexicographic comparison lesser < greater
     return or_(
         *[
             and_(
@@ -97,11 +91,21 @@ def where_condition_for_page(
             "Page marker has different column count to query's order clause"
         )
 
+    native_tuple = (
+        len(place) > 1 and dialect.name.lower() in SUPPORTS_NATIVE_TUPLE_COMPARISON
+    )
+
     zipped = zip(ordering_columns, place)
-    swapped = [c.pair_for_comparison(value, dialect) for c, value in zipped]
+    swapped = [
+        c.pair_for_comparison(value, dialect, apply_bind_processor=native_tuple)
+        for c, value in zipped
+    ]
     row, place_row = zip(*swapped)
 
-    return compare_tuples(greater=row, lesser=place_row, dialect=dialect)
+    if native_tuple:
+        return tuple_(*place_row) < tuple_(*row)
+    else:
+        return compare_tuples(lesser=place_row, greater=row)
 
 
 class _PagingQuery(NamedTuple):
@@ -260,7 +264,7 @@ def orm_get_page(
 
 
 def core_get_page(
-    s: Union[Session, Connection],
+    s: Session,
     selectable: Select[_TP],
     per_page: int,
     place: Optional[Keyset],
@@ -406,7 +410,8 @@ def select_page(
     """
     place, backwards = process_args(after, before, page)
 
-    return core_get_page(s, selectable, per_page, place, backwards)
+    session = get_session(s)
+    return core_get_page(session, selectable, per_page, place, backwards)
 
 
 def get_page(
