@@ -1,6 +1,7 @@
 """Methods for messing with the internals of SQLAlchemy 1.4/2.0 results."""
 from __future__ import annotations
-from sqlalchemy.engine.result import result_tuple
+
+from sqlalchemy.engine.result import SimpleResultMetaData, result_tuple
 from sqlalchemy.engine.row import Row
 
 from .constants import ORDER_COL_PREFIX
@@ -59,7 +60,7 @@ def orm_coerce_row(row, extra_columns, result_type):
 def core_result_type(selectable, s):
     """Given a SQLAlchemy Core selectable and a connection/session, get the
     type constructor for the result row type."""
-    # Unused in sqlalchemy 1.4: see core_coerce_row implementation
+    # Unused in sqlalchemy 1.4+: see core_coerce_row implementation
     return None
 
 
@@ -96,16 +97,37 @@ def core_coerce_row(row: Row, extra_columns, result_type) -> Row:
         return row
     N = len(row) - len(extra_columns)
 
+    parent = row._parent
+    if isinstance(parent, SimpleResultMetaData):
+        parent = parent._reduce(list(parent.keys)[:N])
+    processors = None  # Processors are applied immediately in sqla1.4+
+    data = row._data[:N]
+
+    if hasattr(row, "_key_to_index"):
+        # 2.0.11+
+        structure = (
+            {  # Strip out added OCs from the keymap:
+                k: v
+                for k, v in row._key_to_index.items()
+                if not (isinstance(k, str) and k.startswith(ORDER_COL_PREFIX))
+            },
+        )
+    else:
+        # <2.0.11
+        structure = (
+            {  # Strip out added OCs from the keymap:
+                k: v
+                for k, v in row._keymap.items()
+                if not (isinstance(v[1], str) and v[1].startswith(ORDER_COL_PREFIX))
+            },
+            row._key_style,
+        )
+
     return TruncatedRow(
-        row._parent,
-        None,  # Processors are applied immediately in sqla1.4+
-        {  # Strip out added OCs from the keymap:
-            k: v
-            for k, v in row._keymap.items()
-            if not (isinstance(v[1], str) and v[1].startswith(ORDER_COL_PREFIX))
-        },
-        row._key_style,
-        row._data[:N],
+        parent,
+        processors,
+        *structure,
+        data,
     )
 
 
